/**
 * Utility for working with a PFS file (filesystem)
 * (c) 2012 Peter Lekensteyn <lekensteyn@gmail.com>
 * License: GPLv3+
 */
#include <stdio.h>
#include <unistd.h>
#include <stdint.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>

#include <sys/stat.h>
#include <sys/types.h>

#include "pfs.h"

void print_usage(char **argv) {
	fprintf(stderr, "Usage: %s [-x | -t] [-v] [-C CHDIR] file\n", *argv);
	exit(-1);
}

enum {
	MODE_LIST,
	MODE_EXTRACT,
};

int main(int argc, char **argv) {
	int mode = MODE_LIST;
	int verbose = 0;
	char *filename, *cdir = NULL;
	int i, opt;
	uint16_t file_count;
	while ((opt = getopt(argc, argv, "txvC:")) != -1) {
		switch (opt) {
		case 't':
			mode = MODE_LIST;
			break;
		case 'x':
			mode = MODE_EXTRACT;
			break;
		case 'v':
			verbose = 1;
			break;
		case 'C':
			cdir = strdup(optarg);
			if (!cdir) {
				fprintf(stderr, "Memory alloc failure for -C\n");
				return 1;
			}
			break;
		default:
			print_usage(argv);
		}
	}
	if (optind >= argc) {
		print_usage(argv);
	}
	filename = argv[optind];

	if (cdir) {
		/* try to create, ignore errors */
		mkdir(cdir, 0777);
		if (chdir(cdir)) {
			perror("chdir");
			return 1;
		}
	}

	FILE *fp = fopen(filename, "r");
	if (!fp) {
		perror("opening file");
		return 1;
	}
	struct flist *file = NULL;
	switch (pfs_parse_file(fp, &file_count, &file)) {
		case 2:
			perror("Searching for length");
			return 1;
		case 3:
			perror("fread");
			return 1;
		case 4:
			perror("memory allocation");
			return 1;
	}
	for (i=0x10; file; file=file->next, i+=0x48) {
		if (mode == MODE_LIST) {
			if (verbose) {
				printf("%8x %08X %08X %08X %s\n", i,
					file->file.a,
					file->file.file_offset,
					file->file.size,
					file->file.filename
					);
			} else {
				printf("%s\n", file->file.filename);
			}
		}
		if (mode == MODE_EXTRACT) {
			if (verbose) {
				printf("%s\n", file->file.filename);
			}
			if (pfs_extract(fp, &file->file)) {
				fprintf(stderr, "Error extracting %s\n",
					file->file.filename);
			}
		}
	}
	fclose(fp);
	return 0;
}
